/*
 * simple touch dimmer using an ATtiny13
 *
 * PB4 = touch sensor
 * PB1 = LED driver (H=on)
 *
 * (w) Axel XL Schwenke <axel.schwenke@gmx.de>
 *
 * $Id: main.c 345 2016-06-12 16:06:39Z schwenke $
 */

#include <avr/io.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include "TinyTouchLib.h"


/* defines */

/* next 4 are 0/1 switches */
#define TURN_ON_MAX_BRIGHT     1    // set max brightness when turning light on
#define AUTO_REVERSE_DIR       0    // reverse dimm dir at min/max level
#define REVERSE_DIR_ON_RELEASE 1    // reverse dimm dir when releasing the button
#define ENABLE_WATCHDOG        1    // enable the hardware watchdog

/* next 2 are number of 10ms cycles */
#define DIMM_START_CYCLES      50   // after how many cycles start dimming
#define DIMM_REPEAT_CYCLES     10   // after how many more cycles continue dimming

#if (DIMM_START_CYCLES > 255)
# error DIMM_START_CYCLES must fit in uint8_t
#endif

#if (DIMM_REPEAT_CYCLES > DIMM_START_CYCLES)
# error DIMM_REPEAT_CYCLES cannot be higher than DIMM_START_CYCLES
#endif


/* global constants */

/* dimm levels */
#include "dimm_table.inc"


/* global variables */
uint8_t light_is_on = 0;          // start with lights off
uint8_t dimm_level  = MAX_LEVEL;  // set maximum brightness
int8_t  dimm_dir    = -1;         // dimming starts downwards


/* initialize the hardware */
static void hw_init(void)
{
    // set clock prescaler to 1
    CLKPR = _BV(CLKPCE);
    CLKPR = 0;

    // enable output pin & set to L
    DDRB = _BV(PB1);
    PORTB  &= ~_BV(PB1);

    // Timer0: fast PWM (mode 3)
    // prescaler=256 => overflow @ 147Hz
    TCCR0A = (_BV(WGM01) | _BV(WGM00));
    TCCR0B = (_BV(CS02));
    OCR0B  = dimm_step[dimm_level];

#if (ENABLE_WATCHDOG == 1)
    wdt_enable(WDTO_120MS);
#endif
}


static void light_on(void)
{
    TCCR0A |= _BV(COM0B1);   // enable PWM @ OCR0B = PB1
    light_is_on = 1;
}


static void light_off(void)
{
    TCCR0A &= ~_BV(COM0B1);  // disconnect OCR0B from PB1
    PORTB  &= ~_BV(PB1);     // force output L
    light_is_on = 0;
}


static void light_toggle(void)
{
    if (light_is_on) {
        light_off();
    } else {
#if (TURN_ON_MAX_BRIGHT == 1)
        dimm_level = MAX_LEVEL;
        OCR0B = dimm_step[dimm_level];
        dimm_dir = -1;
#endif
        light_on();
    }
}


/* dimm lights one step according to dimm_dir
 * eventually reverse direction at min/max level */
static void dimm_one_step(void)
{
    if (dimm_dir == 1) {
        if (dimm_level == MAX_LEVEL) {
#if (AUTO_REVERSE_DIR == 1)
            dimm_dir = -1;
#else
            return;
#endif
        }
    } else {
        if (dimm_level == 0) {
#if (AUTO_REVERSE_DIR == 1)
            dimm_dir = 1;
#else
            return;
#endif
        }
    }
    dimm_level += dimm_dir;
    OCR0B = dimm_step[dimm_level];
}


int main(void)
{
    uint8_t touched = 0;  // count cycles while button touched
    uint8_t dimmed = 0;   // remember if we dimmed or not

    hw_init();
    tinytouch_init();

    while (1) {

        switch (tinytouch_sense()) {

        case tt_on:
        case tt_push:
            ++touched;
            if (touched == DIMM_START_CYCLES) {
                if (!light_is_on) {
                    light_on();
                }
                dimm_one_step();
                touched = DIMM_START_CYCLES - DIMM_REPEAT_CYCLES;
                dimmed = 1;
            }
            break;

        case tt_release:
            if (dimmed) {
#if (REVERSE_DIR_ON_RELEASE == 1)
                dimm_dir = -dimm_dir;
#endif
            } else {
                light_toggle();
            }
            touched = 0;
            dimmed = 0;
        }

        _delay_ms(10);

#if (ENABLE_WATCHDOG == 1)
        wdt_reset();
#endif

    }
}

